import 'dart:async';

import 'package:kohana/src/model/event_bus.dart';
import 'package:kohana/src/model/view_state.dart';
import 'package:kohana/src/ui/lifecycle.dart';
import 'package:rxdart/rxdart.dart';

class BaseModel implements Disposable{
  final BehaviorSubject<ViewState> _activity;

  Stream<bool> get isWaiting =>
      _activity.map((activity) => activity == ViewState.Busy);

  Stream<bool> get hasError =>
      _activity.map((activity) => activity == ViewState.Error);

  Stream<bool> get hasData => _activity
      .where((a) => [ViewState.Done, ViewState.Error].contains(a))
      .scan((dynamic acc, curr, i) => curr == ViewState.Done, false)
      .startWith(_activity.value == ViewState.Done);

  BaseModel() : this._activity = BehaviorSubject.seeded(ViewState.Idle);

  void setActivity(ViewState activity) {
    _activity.add(activity);
  }

  @override
  FutureOr dispose() {
    _activity.close();
  }
}
