import 'package:flutter/material.dart';
import 'package:kohana/src/ui/lifecycle.dart';
import 'package:kohana/src/ui/observer.dart';
import 'package:kohana/src/ui/stateful_view.dart';
import 'package:rxdart/rxdart.dart';

// ignore: must_be_immutable
abstract class ReactiveButton extends StatefulView<_Fields> {
  final Func? action;
  final Duration minimumActivityTime;

  ReactiveButton({
    Func? onPressed,
    this.minimumActivityTime = const Duration(milliseconds: 200),
  }) : this.action = onPressed;

  Widget buildButton(BuildContext context, bool isActive);

  @override
  Widget buildWidget(BuildContext context) => Observer<bool>(
        onSuccess: (context, isActive) => buildButton(context, isActive),
        stream: $.isActive.stream,
      );

  Func _userAction(Func? callback) => () async {
        if (callback == null) return;
        try {
          $.isActive.add(true);
          await callback();
          await Future.delayed(minimumActivityTime);
        } catch (e) {
          throw e;
        } finally {
          if (!$.isActive.isClosed) {
            $.isActive.add(false);
          }
        }
      };

  Future<void> onPressed() async {
    await _userAction(action)();
  }

  @override
  FieldsFactory<_Fields> setup() => () => _Fields();
}

class _Fields extends LifecycleAware {
  final isActive = BehaviorSubject.seeded(false);

  @override
  void dispose() {
    isActive.close();
  }
}
