import 'package:kohana/kohana.dart';
import 'package:rxdart/rxdart.dart';

typedef Stream<InputValue> InputStreamBuilder<T>(Stream<T> stream);

abstract class BaseInputField<T> {
  final BehaviorSubject<T> subject;
  final TextFieldValidator? validator;
  final Stream<InputValue>? inputStream;

  BaseInputField(
    this.subject,
    this.validator, {
    InputStreamBuilder<T>? inputStreamBuilder,
  }) : this.inputStream = inputStreamBuilder != null ? inputStreamBuilder(subject.stream).shareValue() : null;

  Stream<InputValue> get asInputValueStream =>
      inputStream ??
      (validator != null
          ? streamAsString.map((value) => validator!.validator!.isValid(value)
              ? InputValue(value)
              : InputValue(value, validationMessage: validator!.errorText))
          : streamAsString.map((event) => InputValue(event)));

  Future close() => subject.close();

  void add(T value) => subject.add(value);

  T? get value => subject.value;

  Stream<T> get stream => subject.stream.distinct();

  Stream<String> get streamAsString => throw "Not implemented";

  Consumer<InputValue> get asInputValueConsumer => throw "Not implemented";
}
