import 'package:flutter/material.dart';
import 'package:kohana/kohana.dart';

import 'base_input_fields.dart';
import 'base_widget_options.dart';

// ignore: must_be_immutable
class BaseInputWidget extends StatefulView<BaseInputFields> {
  final BaseInputWidgetOptions options;

  InputDecoration get inputDecoration => options.inputDecorationBuilder != null
      ? options.inputDecorationBuilder!(defaultDecoration, $)
      : defaultDecoration;

  TextField get textField =>
      options.textFieldBuilder != null ? options.textFieldBuilder!(defaultTextField, $) : defaultTextField;

  TextStyle get textStyle =>
      options.textStyleBuilder != null ? options.textStyleBuilder!(defaultTextStyle, $) : defaultTextStyle;

  InputDecoration get defaultDecoration =>
      defaultDecorationBuilder != null ? defaultDecorationBuilder!(_defaultDecoration, $) : _defaultDecoration;
  static InputDecorationBuilder? defaultDecorationBuilder;

  InputDecoration get _defaultDecoration => InputDecoration(
        errorText: options.showErrorLabel
            ? ($.validationMessage ?? "").isEmpty
                ? null
                : $.validationMessage
            : null,
        focusedBorder: UnderlineInputBorder(
          borderSide: BorderSide(color: Colors.purple, width: 1.0),
        ),
        hintText: options.placeHolder,
        // labelStyle: TextStyle(color: inputLabelColor),
        labelText: options.label,
        errorStyle: TextStyle(
            // fontSize: 16.0,
            ),
        labelStyle: TextStyle(
          fontSize: 18.0,
          color: Color(0xff7A779E),
        ),
      );

  TextStyle get defaultTextStyle =>
      defaultTextStyleBuilder != null ? defaultTextStyleBuilder!(_defaultTextStyle, $) : _defaultTextStyle;
  static TextStyleBuilder? defaultTextStyleBuilder;

  TextStyle get _defaultTextStyle => TextStyle(
        color: Colors.blueGrey,
        fontWeight: FontWeight.w400,
        fontSize: 18,
      );

  static TextFieldBuilder? defaultTextFieldBuilder;

  TextField get defaultTextField =>
      defaultTextFieldBuilder != null ? defaultTextFieldBuilder!(_defaultTextField, $) : _defaultTextField;

  TextField get _defaultTextField => TextField(
      keyboardType: options.type,
      readOnly: options.readOnly,
      textAlign: options.textAlign,
      style: textStyle,
      decoration: inputDecoration,
      controller: $.controller,
      focusNode: options.focusNode,
      autofocus: options.autofocus ?? false,
      inputFormatters: options.inputFormatters ?? [],
      obscureText: options.obscureText,
      onSubmitted: (val) {
        if (options.nextFocusNode != null) {
          _fieldFocusChange(context, options.focusNode!, options.nextFocusNode!);
        }
      });

  BaseInputWidget({this.options = const BaseInputWidgetOptions()});

  @override
  Widget buildWidget(BuildContext context) => Observer<_ValidationResult>(
        stream: $.asStream().map((event) => _ValidationResult(event)),
        onSuccess: (_, validationMessage) => textField,
      );

  _fieldFocusChange(BuildContext context, FocusNode currentFocus, FocusNode nextFocus) {
    currentFocus.unfocus();
    FocusScope.of(context).requestFocus(nextFocus);
  }

  @override
  FieldsFactory<BaseInputFields> setup() => () => BaseInputFields(options);
}

class _ValidationResult {
  final String? validationMessage;

  _ValidationResult(this.validationMessage);
}
