import 'dart:async';

// import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:kohana/kohana.dart';

import 'base_input_fields.dart';

// part 'base_widget_options.g.dart';

typedef InputDecoration InputDecorationBuilder(
    InputDecoration inputDecoration, BaseInputFields $);
typedef TextField TextFieldBuilder(TextField textField, BaseInputFields $);
typedef TextStyle TextStyleBuilder(TextStyle textStyle, BaseInputFields $);
typedef BaseInputWidgetStateWatcher = StreamSubscription Function(
    BaseInputFields);
typedef BaseInputWidgetOnMounted = void Function(BaseInputFields);

// @CopyWith()
class BaseInputWidgetOptions {
  final InputDecorationBuilder? inputDecorationBuilder;
  final TextFieldBuilder? textFieldBuilder;
  final TextStyleBuilder? textStyleBuilder;
  final Stream<InputValue>? inputValueStream;
  final Consumer<InputValue>? changeListener;
  final BaseInputWidgetStateWatcher? watcher;
  final BaseInputWidgetOnMounted? onMounted;
  final Stream<Signal>? validationStream;

  final TextEditingController? controller;
  final String? value;
  final List<TextInputFormatter>? inputFormatters;
  final FocusNode? nextFocusNode;
  final FocusNode? focusNode;
  final bool? autofocus;
  final TextAlign textAlign;
  final bool showErrorLabel;

  final String? label;
  final String? placeHolder;
  final TextInputType? type;
  final bool obscureText;
  final bool readOnly;
  final bool enabled;
  final TextStyle? textStyle;
  final bool? onFocusRemoveError;

  static BaseInputWidgetOptions defaultInstance =
      BaseInputWidgetOptions(onFocusRemoveError: true);

  const BaseInputWidgetOptions({
    this.inputDecorationBuilder,
    this.textFieldBuilder,
    this.textStyleBuilder,
    this.focusNode,
    this.nextFocusNode,
    this.inputValueStream,
    this.changeListener,
    this.watcher,
    this.onMounted,
    this.controller,
    this.value,
    this.inputFormatters,
    this.autofocus,
    this.textAlign = TextAlign.start,
    this.label,
    this.placeHolder,
    this.type,
    this.obscureText = false,
    this.readOnly = false,
    this.showErrorLabel = true,
    this.enabled = true,
    this.textStyle,
    this.validationStream,
    this.onFocusRemoveError,
  });

  @override
  String toString() {
    return 'BaseInputWidgetOptions{inputDecorationBuilder: $inputDecorationBuilder, textFieldBuilder: $textFieldBuilder, focusNode: $focusNode, nextFocusNode: $nextFocusNode, inputValueStream: $inputValueStream, changeListener: $changeListener, controller: $controller}';
  }

  BaseInputWidgetOptions copyWith({
    bool? autofocus,
    void Function(InputValue)? changeListener,
    TextEditingController? controller,
    bool? enabled,
    FocusNode? focusNode,
    InputDecoration Function(InputDecoration, BaseInputFields)?
    inputDecorationBuilder,
    List<TextInputFormatter>? inputFormatters,
    Stream<InputValue>? inputValueStream,
    String? label,
    FocusNode? nextFocusNode,
    bool? obscureText,
    bool? onFocusRemoveError,
    void Function(BaseInputFields)? onMounted,
    String? placeHolder,
    bool? readOnly,
    bool? showErrorLabel,
    TextAlign? textAlign,
    TextField Function(TextField, BaseInputFields)? textFieldBuilder,
    TextStyle? textStyle,
    TextStyle Function(TextStyle, BaseInputFields)? textStyleBuilder,
    TextInputType? type,
    Stream<Signal>? validationStream,
    String? value,
    StreamSubscription<dynamic> Function(BaseInputFields)? watcher,
  }) {
    return BaseInputWidgetOptions(
      autofocus: autofocus ?? this.autofocus,
      changeListener: changeListener ?? this.changeListener,
      controller: controller ?? this.controller,
      enabled: enabled ?? this.enabled,
      focusNode: focusNode ?? this.focusNode,
      inputDecorationBuilder:
      inputDecorationBuilder ?? this.inputDecorationBuilder,
      inputFormatters: inputFormatters ?? this.inputFormatters,
      inputValueStream: inputValueStream ?? this.inputValueStream,
      label: label ?? this.label,
      nextFocusNode: nextFocusNode ?? this.nextFocusNode,
      obscureText: obscureText ?? this.obscureText,
      onFocusRemoveError: onFocusRemoveError ?? this.onFocusRemoveError,
      onMounted: onMounted ?? this.onMounted,
      placeHolder: placeHolder ?? this.placeHolder,
      readOnly: readOnly ?? this.readOnly,
      showErrorLabel: showErrorLabel ?? this.showErrorLabel,
      textAlign: textAlign ?? this.textAlign,
      textFieldBuilder: textFieldBuilder ?? this.textFieldBuilder,
      textStyle: textStyle ?? this.textStyle,
      textStyleBuilder: textStyleBuilder ?? this.textStyleBuilder,
      type: type ?? this.type,
      validationStream: validationStream ?? this.validationStream,
      value: value ?? this.value,
      watcher: watcher ?? this.watcher,
    );
  }

}
