import 'package:kohana/kohana.dart';
import 'package:rxdart/rxdart.dart';

mixin ValidationMixin on BaseViewModel {
  final validationMessages = BehaviorSubject<List<InputValue>>.seeded([]);
  final _validationStreamSubject = PublishSubject<Signal>();

  Stream<Signal> get validationStream => _validationStreamSubject.stream;

  List<String> get errorValidationMessages => (validationMessages.value!)
      .where((element) => element.validationMessage != null)
      .map((e) => e.validationMessage!)
      .toList(growable: false);

  String get errorValidationMessagesString => errorValidationMessages.length ==
          0
      ? ""
      : errorValidationMessages.reduce((value, element) => "$value\n$element");

  final BehaviorSubject<bool> isValid = BehaviorSubject.seeded(false);

  bool get isValidValue => isValid.value!;

  void triggerValidation() => _validationStreamSubject.add(Signal());

  @override
  void start() {
    super.start();
    disposeBag.addAll([
      validationMessages
          .map((event) => event
              .where((element) => element.validationMessage != null)
              .toList(growable: false))
          .map(($) => $.length == 0)
          .listen(isValid.add),
    ]);
  }

  @override
  void dispose() {
    isValid.close();
    validationMessages.close();
    _validationStreamSubject.close();
    super.dispose();
  }
}

extension AsValidatedInputValueExt on Stream<String> {
  Stream<InputValue> asValidatedInputValue(
          TextFieldValidator textFieldValidator) =>
      this.distinct().map((value) => textFieldValidator.validator!
              .isValid(value)
          ? InputValue(value)
          : InputValue(value, validationMessage: textFieldValidator.errorText));
}
